#program v pythonu 3.8 na fitovani velikosti vodivosti RLV obvodu. Vykresluje take fazi. Je potreba nainstalovat balicky numpy a lmfit
import matplotlib.pyplot as plt   #nacteni knihovny pro kresleni
import numpy as np    #nacteni numericke knihovny numpy
from lmfit import Minimizer, Parameters, fit_report   #nacteni knihovny pro fitovani LMfit
from uncertainties import ufloat, correlated_values #balicek na vypocet propagace chyb


#definice funkci
def Gabs(R,L,C,o):   #absolutni velikost vodivosti 
	return 1/np.sqrt(R**2+(o*L-1/(o*C))**2)
def Gphi(R,L,C,o):  #faze ve stupnich
	return np.arctan(-(o*L-1/(o*C))/R)/np.pi*180
def residual(pars, o, Gdata):    #definice residualu: rozdilu fitovane funkce a dat
	return Gabs(pars['R'],pars['L'],pars['C'],o) - Gdata  #zde se fituje jen rozdil absolutni hodnoty vodivosti mezi teorii a daty, ne faze

#nacteni dat
data=np.loadtxt('RLCfrekv.dat', skiprows=1)   #nacteni exprimentalnich dat do pole "data", preskakuje prvni radek
fdata=data[:,0]   #nacte frekvenci z prvniho sloupce
odata=fdata*2*np.pi  #prepocteni na kruhovou frekvenci
Gdata=data[:,1]  #nacte amplitudy vodivosti z druheho sloupce 
Phase=data[:,2]  #nacteni amplitudu faze z tretiho sloupce 

#definice fitovaich parametru
ParsStart = Parameters()   #definice objektu fitovacich parametru 
#zavedeni jednotlivych fitovacich parametru se startovaci fitovaci hodnotou a informaci, zda se fituje nebo ne (vary=True/False)
#pri nelinearnim fitovani by mely byt startovaci hodnoty parametru relativne blizko konecnym, jinak fit nemusi dobre zkonvergovat
ParsStart.add('R', value=30,vary=True)   #hodnota odporu
ParsStart.add('L', value=0.12,vary=True)  #hodnota indukce
ParsStart.add('C', value=217E-9,vary=True)  #hodnota kapacity

#fitovani
minner = Minimizer(residual, ParsStart, fcn_args=(odata,Gdata))  #samotne fitovani. Vola funkci residual, pouziva parametrky k fitovani Parstart
results = minner.minimize()  #ulozeni vysledku fitovani do promenne result
ParsFit= results.params   #ulozeni vyslednych hodnot fitovacich parametru do promenne ParsFit

#vypocet startovacich a fitovacich funkci
ftheor = np.arange(np.amin(fdata), np.amax(fdata), 1)  # vytvoreni osy frekvence pro teoreticky vypocet v ramci merenych dat s krokem 1 Hz
otheor=ftheor*2*np.pi   #prepcet na kruhovou frekvenci
# vypocet startovaci funkce Gabs (v nelinerarnim fitovani by nemela by byt moc daleko od experimentalnich dat
GabsStart = Gabs(ParsStart["R"],ParsStart["L"],ParsStart["C"],otheor)  
GphiStart = Gphi(ParsStart["R"],ParsStart["L"],ParsStart["C"],otheor)  # vypocet startovaci funkce Gphi
GabsFit = Gabs(ParsFit["R"],ParsFit["L"],ParsFit["C"],otheor)  # vypocet nafitovane funkce Gabs
GphiFit = Gphi(ParsFit["R"],ParsFit["L"],ParsFit["C"],otheor)  # vypocet nafitovane funkce Gphi

#vypsani statistiky na obrazovku a ulozeni do souboru
print(fit_report(results))  # write error report, alternative result.params.pretty_print()
FileStatistika= open('Statistika.dat', 'w+')  #ulozeni statistiky do souboru
print(fit_report(results),file=FileStatistika)
FileStatistika.close()

#ulozeni startovaci a nafitovanych zavislosti do souboru
FileFitSpekta= open('FitSpekta.dat', 'w+')
print ("f[Hz]", "\t", "GabsStart",  "\t", "GphiStart", "\t", "GabsFit",  "\t", "GphiFit", file=FileFitSpekta)
for i in range(len(ftheor)):
    print (ftheor[i],"\t",GabsStart[i],  "\t",GphiStart[i], "\t", GabsFit[i],  "\t",GphiFit[i], file=FileFitSpekta)
FileFitSpekta.close()

#vypocet dalsich pozadovanych hodnot parametru z obdrzenych hodnot R,L,C
#vytvoreni promennych ufloat z knihovny uncertainties pro vypocet nejen hodnot, ale i chyb. Prvni parametr je hodnota, druhy chyba
#POZOR, ZAKON SIRENI CHYB V TETO PODOBE PREDPOKLADA NEKORELOVANE PARAMETRY, COZ C A L JSOU
R=ufloat(ParsFit['R'], ParsFit['R'].stderr)  
L=ufloat(ParsFit['L'], ParsFit['L'].stderr)
C=ufloat(ParsFit['C'], ParsFit['C'].stderr)
# vypocet dalcich hodnot. Knihovna uncertainties k tomu automaticky vypocte chyby dle zakona sireni chyb 


omega0=1/((L*C)**0.5)
f0=omega0/(2*np.pi)
print("Hodnoty spoctene na zaklade zakona sireni chyb pro NEKORELOVANE promenne")
print("f0=", f0,"omega0=", omega0)
F=1/L
alpha=R/(2*L)
Q=omega0/(2*alpha)
print("F=", F,"alpha=", alpha, "Q=", Q)

vals=[ParsFit[a].value for a in results.var_names]
Rcor,Lcor,Ccor=correlated_values(vals,results.covar)

omega0cor=1/((Lcor*Ccor)**0.5)
f0cor=omega0cor/(2*np.pi)
print("Hodnoty spoctene na zaklade zakona sireni chyb pro KORELOVANE promenne")
print("f0=", f0cor,"omega0=", omega0cor)
Fcor=1/Lcor
alphacor=Rcor/(2*Lcor)
Qcor=omega0cor/(2*alphacor)
print("F=", F,"alpha=", alpha, "Q=", Q)



#vykresleni obrazku amplitudy a faze
fig, (ax1, ax2) = plt.subplots(1, 2)  
#levy obrazek amplitudy
ax1.plot(ftheor,GabsStart,'k-',label='|G| Start')
ax1.plot(ftheor,GabsFit,'b-',label=' |G| Fit')
ax1.plot(fdata,Gdata,'r*',label='|G| Data')
ax1.legend(loc='upper right', shadow=False)
ax1.set(xlabel='f [Hz]', ylabel='amplituda G [Ohm-1]')
#pravy obrazek faze
ax2.plot(ftheor,GphiStart,'k-',label='Faze Start')
ax2.plot(ftheor,GphiFit,'b-',label='Faze Fit')
ax2.plot(fdata,Phase,'r*',label='Phase Data')
ax2.legend(loc='upper right', shadow=False)
ax2.set(xlabel='f [Hz]', ylabel='faze [deg]')
plt.show()

